package com.eksad.masterdata.domain;

import com.eksad.ddms.common.util.object.EntityObject;

import jakarta.persistence.*;
import java.util.HashSet;
import java.util.Objects;
import java.util.Set;

@Entity
@Table(name= "MST_PROMOWORKORDER")
public class PromoWorkOrder implements EntityObject<PromoWorkOrder> {

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE)
    Integer id;

    private String promoWoID;
    private String promoID;
    private String promoName;
    private Boolean joinable;
    private Double value;
    private String woID;
    private Boolean isManualPromo;

    @OneToMany(fetch = FetchType.EAGER, cascade = CascadeType.ALL, orphanRemoval = true)
    @JoinColumn(name = "promoWoID", referencedColumnName = "promoWoID")
    private Set<PromoWorkOrderDetail> promoWoDetails = new HashSet<>();

    @Embedded
    private CreationalSpecification promoWoCreational;

    public PromoWorkOrder() {
    }

    public PromoWorkOrder(String promoWoID, String promoID, Boolean isManualPromo, String woID, CreationalSpecification promoWoCreational, Set<PromoWorkOrderDetail> promoWoDetails) {
        this.promoWoID = promoWoID;
        this.promoID = promoID;
        this.woID = woID;
        this.isManualPromo = isManualPromo;
        this.promoWoCreational = promoWoCreational;
        this.promoWoDetails = promoWoDetails;
    }

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public String getPromoWoID() {
        return promoWoID;
    }

    public void setPromoWoID(String promoWoID) {
        this.promoWoID = promoWoID;
    }

    public String getPromoID() {
        return promoID;
    }

    public void setPromoID(String promoID) {
        this.promoID = promoID;
    }

    public String getWoID() {
        return woID;
    }

    public void setWoID(String woID) {
        this.woID = woID;
    }

    public Boolean getIsManualPromo() {
        return isManualPromo;
    }

    public void setIsManualPromo(Boolean isManualPromo) {
        this.isManualPromo = isManualPromo;
    }

    public CreationalSpecification getPromoWoCreational() {
        return promoWoCreational;
    }

    public void setPromoWoCreational(CreationalSpecification promoWoCreational) {
        this.promoWoCreational = promoWoCreational;
    }

    public Boolean getJoinable() {
        return joinable;
    }

    public void setJoinable(Boolean joinable) {
        this.joinable = joinable;
    }

    public Double getValue() {
        return value;
    }

    public void setValue(Double value) {
        this.value = value;
    }

    public String getPromoName() {
        return promoName;
    }

    public void setPromoName(String promoName) {
        this.promoName = promoName;
    }

    public Set<PromoWorkOrderDetail> getPromoWoDetails() {
        return promoWoDetails;
    }

    public void setPromoWoDetails(Set<PromoWorkOrderDetail> promoWoDetails) {
        this.promoWoDetails = promoWoDetails;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 71 * hash + Objects.hashCode(this.id);
        hash = 71 * hash + Objects.hashCode(this.promoWoID);
        hash = 71 * hash + Objects.hashCode(this.promoID);
        hash = 71 * hash + Objects.hashCode(this.promoName);
        hash = 71 * hash + Objects.hashCode(this.joinable);
        hash = 71 * hash + Objects.hashCode(this.value);
        hash = 71 * hash + Objects.hashCode(this.woID);
        hash = 71 * hash + Objects.hashCode(this.isManualPromo);
        hash = 71 * hash + Objects.hashCode(this.promoWoCreational);
        hash = 71 * hash + Objects.hashCode(this.promoWoDetails);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final PromoWorkOrder other = (PromoWorkOrder) obj;
        if (!Objects.equals(this.promoWoID, other.promoWoID)) {
            return false;
        }
        if (!Objects.equals(this.promoID, other.promoID)) {
            return false;
        }
        if (!Objects.equals(this.promoName, other.promoName)) {
            return false;
        }
        if (!Objects.equals(this.woID, other.woID)) {
            return false;
        }
        if (!Objects.equals(this.isManualPromo, other.isManualPromo)) {
            return false;
        }
        if (!Objects.equals(this.id, other.id)) {
            return false;
        }
        if (!Objects.equals(this.joinable, other.joinable)) {
            return false;
        }
        if (!Objects.equals(this.value, other.value)) {
            return false;
        }
        if (!Objects.equals(this.promoWoCreational, other.promoWoCreational)) {
            return false;
        }
        if (!Objects.equals(this.promoWoDetails, other.promoWoDetails)) {
            return false;
        }
        return true;
    }

    @Override
    public boolean sameIdentityAs(PromoWorkOrder t) {
        return this.equals(t);
    }
}
