package com.eksad.masterdata.domain;

import com.eksad.ddms.common.h2.type.servicelocation.ServiceLocationType;
import com.eksad.ddms.common.type.*;
import com.eksad.ddms.common.util.object.EntityObject;

import jakarta.persistence.*;

import javax.validation.constraints.NotNull;
import java.sql.Time;
import java.util.Date;
import java.util.HashSet;
import java.util.Objects;
import java.util.Set;

@Entity
@Table(name = "TRX_WO")
public class WorkOrder implements EntityObject<WorkOrder> {

    @Id
    @GeneratedValue(generator = "trx_wo_gen")
    @SequenceGenerator(name = "trx_wo_gen", sequenceName = "trx_wo_seq")
    Integer id;

    @Version
    @Column(name = "version")
    private Long version;

    @NotNull(message="woID cannot be null")
    private String woID;

    @NotNull(message="ahassID cannot be null")
    private String ahassID;

    private Time estimasiMulai;
    private Time estimasiSelesai;
    private String noPolisi;
    private String woNoRangka;
    private String woNoMesin;
    private String unitCustomerID;
    private String unitCustomerName;
    private String customerAttendeeID;
    private Integer kmMotor;
    private Integer suggestedKm;
    private Double fuelMeter;
    private String alasanDatang;
    @OneToMany(fetch = FetchType.EAGER, cascade = CascadeType.ALL, orphanRemoval = true)
    @JoinColumn(name = "woID", referencedColumnName = "woID")
    private Set<ComplaintWorkOrder> listComplaint =  new HashSet<>();
    private String analisa;
    private String typeJasa;
    @OneToMany(fetch = FetchType.EAGER, cascade = CascadeType.ALL, orphanRemoval = true)
    @JoinColumn(name = "woID", referencedColumnName = "woID")
    private Set<WorkOrderService> listJasa =  new HashSet<>();
    private Double totalHargaJasa;
    private Double totalDiscountJasa;
    private Integer totalWaktuJasa;
    @OneToMany(fetch = FetchType.EAGER, cascade = CascadeType.ALL, orphanRemoval = true)
    @JoinColumn(name = "woID", referencedColumnName = "woID")
    private Set<WorkOrderSparepart> listPart =  new HashSet<>();
    private Double totalHargaSparepart;
    private Double totalDiscountSparepart;
    @Enumerated(EnumType.STRING)
    private WoState woLastKnownState;
    @Enumerated(EnumType.STRING)
    private WoStatus woLastKnownStatus;
    @Enumerated(EnumType.STRING)
    private WoDisclaimerType woDisclaimerType;
    private Boolean sparepartDibawaCustomer;
    private String confirmNoTelp;
    @OneToMany(fetch = FetchType.EAGER, cascade = CascadeType.ALL, orphanRemoval = true)
    @JoinColumn(name = "woID", referencedColumnName = "woID")
    private Set<PromoWorkOrder> listPromo = new HashSet<>();
    private Integer queueNumber;
    private String mechanicID;
    private String mechanicName;
    private String unitCategoryID;
    private String unitCategoryName;
    private String unitSeriesID;
    private String unitSeriesName;
    private String unitTypeID;
    private String unitTypeName;
    private String unitTypeColor;
    private Integer tahunPerakitan;
    private String saranMekanik;
    @Enumerated(EnumType.STRING)
    private PitType pitType;
    private Integer pitNumber;
    @Temporal(TemporalType.DATE)
    private Date woDate;
    private String signPhotoCustomer;
    private String signPhotoSA;
    private String signPhotoFinalCheck;
    private String signPhotoPenyerahan;
    private String signPhotoTambahan;
    private String packageServiceID;
    //    @Enumerated(EnumType.STRING)
//    private PaymentStatus paymentStatus;
    private String bookingID;
    private String keterangan;
    @Enumerated(EnumType.STRING)
    private HoldWOReasonType alasanHoldWO;
    private String location;
    private String soID;
    private String promoCode;
    private String modifiedTrigger;
    private String invoice;
    private String invoiceUrl;
    private String unitEntrySource;
    private String noBukuC2;
    private String rekomendasiSA;
    @Enumerated(EnumType.STRING)
    private UnitOwnershipStatus unitOwnership;
    private Double totalBookingFee;
    private String tipeKPB;
    private Integer countPrint;
    private String prosesServicePUD;
    private Boolean specialCase;

    private String csID;
    private String reffNumber;
    @Enumerated(EnumType.STRING)
    private ServiceLocationType serviceLocationType;

    @Embedded
    private CreationalSpecification woCreational;
    private String posServiceID;
    private String address;
    private String kabkotID;
    private String kecamatanID;
    private String kelurahanID;
    private String provinceID;

    public WorkOrder() {
    }

    public WorkOrder(String woID, Long version, String ahassID, Time estimasiMulai, Time estimasiSelesai, String noPolisi, String woNoRangka, String woNoMesin, String unitCustomerID, String unitCustomerName, String customerAttendeeID, Integer kmMotor, Integer suggestedKm, Double fuelMeter, String alasanDatang, String analisa, String typeJasa, Double totalHargaJasa, Double totalDiscountJasa, Integer totalWaktuJasa, Double totalHargaSparepart, Double totalDiscountSparepart, WoState woLastKnownState, WoStatus woLastKnownStatus, WoDisclaimerType woDisclaimerType, Boolean sparepartDibawaCustomer, String confirmNoTelp, Integer queueNumber, String mechanicID,
                     String mechanicName, String unitTypeID, String unitTypeName, String unitTypeColor, Integer tahunPerakitan, String saranMekanik, PitType pitType, Integer pitNumber, Date woDate, String signPhotoCustomer, String signPhotoSA, String packageServiceID, String bookingID, String keterangan, String location, String soID, String promoCode, String modifiedTrigger, String invoice, CreationalSpecification woCreational,
                     String unitEntrySource, String noBukuC2, String rekomendasiSA, UnitOwnershipStatus unitOwnership, Double totalBookingFee, String tipeKPB, Integer countPrint, String prosesServicePUD, String posServiceID, String csID, Boolean specialCase,
                     String address, String kabkotID, String kecamatanID, String kelurahanID, String provinceID, String reffNumber, ServiceLocationType serviceLocationType) {

        this.woID = woID;
        this.version = version;
        this.ahassID = ahassID;
        this.estimasiMulai = estimasiMulai;
        this.estimasiSelesai = estimasiSelesai;
        this.noPolisi = noPolisi;
        this.woNoRangka = woNoRangka;
        this.woNoMesin = woNoMesin;
        this.unitCustomerID = unitCustomerID;
        this.unitCustomerName = unitCustomerName;
        this.customerAttendeeID = customerAttendeeID;
        this.kmMotor = kmMotor;
        this.suggestedKm = suggestedKm;
        this.fuelMeter = fuelMeter;
        this.alasanDatang = alasanDatang;
        this.analisa = analisa;
        this.typeJasa = typeJasa;
        this.totalHargaJasa = totalHargaJasa;
        this.totalDiscountJasa = totalDiscountJasa;
        this.totalWaktuJasa = totalWaktuJasa;
        this.totalHargaSparepart = totalHargaSparepart;
        this.totalDiscountSparepart = totalDiscountSparepart;
        this.woLastKnownState = woLastKnownState;
        this.woLastKnownStatus = woLastKnownStatus;
        this.woDisclaimerType = woDisclaimerType;
        this.sparepartDibawaCustomer = sparepartDibawaCustomer;
        this.confirmNoTelp = confirmNoTelp;
        this.queueNumber = queueNumber;
        this.mechanicID = mechanicID;
        this.mechanicName = mechanicName;
        this.unitTypeID = unitTypeID;
        this.unitTypeName = unitTypeName;
        this.unitTypeColor = unitTypeColor;
        this.tahunPerakitan = tahunPerakitan;
        this.saranMekanik = saranMekanik;
        this.pitType = pitType;
        this.pitNumber = pitNumber;
        this.woDate = woDate;
        this.signPhotoCustomer = signPhotoCustomer;
        this.signPhotoSA = signPhotoSA;
        this.packageServiceID = packageServiceID;
        this.bookingID = bookingID;
        this.keterangan = keterangan;
        this.location = location;
        this.soID = soID;
        this.promoCode = promoCode;
        this.modifiedTrigger = modifiedTrigger;
        this.invoice = invoice;
        this.woCreational = woCreational;
        this.noBukuC2 = noBukuC2;
        this.unitEntrySource = unitEntrySource;
        this.rekomendasiSA = rekomendasiSA;
        this.unitOwnership = unitOwnership;
        this.totalBookingFee = totalBookingFee;
        this.tipeKPB = tipeKPB;
        this.countPrint = countPrint;
        this.prosesServicePUD = prosesServicePUD;
        this.specialCase = specialCase;
        this.csID = csID;

        this.posServiceID = posServiceID;
        this.address = address;
        this.kabkotID = kabkotID;
        this.kecamatanID = kecamatanID;
        this.kelurahanID = kelurahanID;
        this.provinceID = provinceID;
        this.reffNumber = reffNumber;
        this.serviceLocationType = serviceLocationType;
    }

    public String getSignPhotoFinalCheck() {
        return signPhotoFinalCheck;
    }

    public void setSignPhotoFinalCheck(String signPhotoFinalCheck) {
        this.signPhotoFinalCheck = signPhotoFinalCheck;
    }

    public String getSignPhotoPenyerahan() {
        return signPhotoPenyerahan;
    }

    public void setSignPhotoPenyerahan(String signPhotoPenyerahan) {
        this.signPhotoPenyerahan = signPhotoPenyerahan;
    }

    public String getSignPhotoTambahan() {
        return signPhotoTambahan;
    }

    public void setSignPhotoTambahan(String signPhotoTambahan) {
        this.signPhotoTambahan = signPhotoTambahan;
    }

    public Long getVersion() {
        return version;
    }

    public void setVersion(Long version) {
        this.version = version;
    }

    public HoldWOReasonType getAlasanHoldWO() {
        return alasanHoldWO;
    }

    public void setAlasanHoldWO(HoldWOReasonType alasanHoldWO) {
        this.alasanHoldWO = alasanHoldWO;
    }

    public Integer getQueueNumber() {
        return queueNumber;
    }

    public void setQueueNumber(Integer queueNumber) {
        this.queueNumber = queueNumber;
    }

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public String getWoID() {
        return woID;
    }

    public void setWoID(String woID) {
        this.woID = woID;
    }

    public String getAhassID() {
        return ahassID;
    }

    public void setAhassID(String ahassID) {
        this.ahassID = ahassID;
    }

    public Time getEstimasiMulai() {
        return estimasiMulai;
    }

    public void setEstimasiMulai(Time estimasiMulai) {
        this.estimasiMulai = estimasiMulai;
    }

    public Time getEstimasiSelesai() {
        return estimasiSelesai;
    }

    public void setEstimasiSelesai(Time estimasiSelesai) {
        this.estimasiSelesai = estimasiSelesai;
    }

    public String getNoPolisi() {
        return noPolisi;
    }

    public void setNoPolisi(String noPolisi) {
        this.noPolisi = noPolisi;
    }

    public String getWoNoRangka() {
        return woNoRangka;
    }

    public void setWoNoRangka(String woNoRangka) {
        this.woNoRangka = woNoRangka;
    }

    public String getWoNoMesin() {
        return woNoMesin;
    }

    public void setWoNoMesin(String woNoMesin) {
        this.woNoMesin = woNoMesin;
    }

    public String getUnitCustomerID() {
        return unitCustomerID;
    }

    public void setUnitCustomerID(String unitCustomerID) {
        this.unitCustomerID = unitCustomerID;
    }

    public String getCustomerAttendeeID() {
        return customerAttendeeID;
    }

    public void setCustomerAttendeeID(String customerAttendeeID) {
        this.customerAttendeeID = customerAttendeeID;
    }

    public Integer getKmMotor() {
        return kmMotor;
    }

    public void setKmMotor(Integer kmMotor) {
        this.kmMotor = kmMotor;
    }

    public Integer getSuggestedKm() {
        return suggestedKm;
    }

    public void setSuggestedKm(Integer suggestedKm) {
        this.suggestedKm = suggestedKm;
    }

    public Double getFuelMeter() {
        return fuelMeter;
    }

    public void setFuelMeter(Double fuelMeter) {
        this.fuelMeter = fuelMeter;
    }

    public String getAlasanDatang() {
        return alasanDatang;
    }

    public void setAlasanDatang(String alasanDatang) {
        this.alasanDatang = alasanDatang;
    }

    public Set<ComplaintWorkOrder> getListComplaint() {
        return listComplaint;
    }

    public void setListComplaint(Set<ComplaintWorkOrder> listComplaint) {
        if(listComplaint == null){
            this.listComplaint = listComplaint;
        }else {
            if(listComplaint.isEmpty()){
                this.listComplaint.addAll(listComplaint);
            }else {
                this.listComplaint.retainAll(listComplaint);
                this.listComplaint.addAll(listComplaint);
            }
        }
    }

    public String getSaranMekanik() {
        return saranMekanik;
    }

    public void setSaranMekanik(String saranMekanik) {
        this.saranMekanik = saranMekanik;
    }

    public String getAnalisa() {
        return analisa;
    }

    public void setAnalisa(String analisa) {
        this.analisa = analisa;
    }

    public String getTypeJasa() {
        return typeJasa;
    }

    public void setTypeJasa(String typeJasa) {
        this.typeJasa = typeJasa;
    }

    public Set<WorkOrderService> getListJasa() {
        return listJasa;
    }

    public void setListJasa(Set<WorkOrderService> listJasa) {
        if(listJasa == null){
            this.listJasa = listJasa;
        }else {
            if(listJasa.isEmpty()){
                this.listJasa.addAll(listJasa);
            }else {
                this.listJasa.retainAll(listJasa);
                this.listJasa.addAll(listJasa);
            }
        }
    }

    public Double getTotalHargaJasa() {
        return totalHargaJasa;
    }

    public void setTotalHargaJasa(Double totalHargaJasa) {
        this.totalHargaJasa = totalHargaJasa;
    }

    public Double getTotalDiscountJasa() {
        return totalDiscountJasa;
    }

    public void setTotalDiscountJasa(Double totalDiscountJasa) {
        this.totalDiscountJasa = totalDiscountJasa;
    }

    public Integer getTotalWaktuJasa() {
        return totalWaktuJasa;
    }

    public void setTotalWaktuJasa(Integer totalWaktuJasa) {
        this.totalWaktuJasa = totalWaktuJasa;
    }

    public Set<WorkOrderSparepart> getListPart() {
        return listPart;
    }

    public void setListPart(Set<WorkOrderSparepart> listPart) {
        if(listPart == null){
            this.listPart = listPart;
        }else {
            if(listPart.isEmpty()){
                this.listPart.clear();
            }else {
                this.listPart.retainAll(listPart);
                this.listPart.addAll(listPart);
            }
        }
    }

    public Double getTotalHargaSparepart() {
        return totalHargaSparepart;
    }

    public void setTotalHargaSparepart(Double totalHargaSparepart) {
        this.totalHargaSparepart = totalHargaSparepart;
    }

    public Double getTotalDiscountSparepart() {
        return totalDiscountSparepart;
    }

    public void setTotalDiscountSparepart(Double totalDiscountSparepart) {
        this.totalDiscountSparepart = totalDiscountSparepart;
    }

    public WoState getWoLastKnownState() {
        return woLastKnownState;
    }

    public void setWoLastKnownState(WoState woLastKnownState) {
        this.woLastKnownState = woLastKnownState;
    }

    public WoStatus getWoLastKnownStatus() {
        return woLastKnownStatus;
    }

    public void setWoLastKnownStatus(WoStatus woLastKnownStatus) {
        this.woLastKnownStatus = woLastKnownStatus;
    }

    public WoDisclaimerType getWoDisclaimerType() {
        return woDisclaimerType;
    }

    public void setWoDisclaimerType(WoDisclaimerType woDisclaimerType) {
        this.woDisclaimerType = woDisclaimerType;
    }

    public Boolean getSparepartDibawaCustomer() {
        return sparepartDibawaCustomer;
    }

    public void setSparepartDibawaCustomer(Boolean sparepartDibawaCustomer) {
        this.sparepartDibawaCustomer = sparepartDibawaCustomer;
    }

    public String getConfirmNoTelp() {
        return confirmNoTelp;
    }

    public void setConfirmNoTelp(String confirmNoTelp) {
        this.confirmNoTelp = confirmNoTelp;
    }

    public Date getWoDate() {
        return woDate;
    }

    public void setWoDate(Date woDate) {
        this.woDate = woDate;
    }

    public Set<PromoWorkOrder> getListPromo() {
        return listPromo;
    }

    public String getMechanicID() {
        return mechanicID;
    }

    public void setMechanicID(String mechanicID) {
        this.mechanicID = mechanicID;
    }

    public String getUnitTypeID() {
        return unitTypeID;
    }

    public void setUnitTypeID(String unitTypeID) {
        this.unitTypeID = unitTypeID;
    }

    public Integer getTahunPerakitan() {
        return tahunPerakitan;
    }

    public void setTahunPerakitan(Integer tahunPerakitan) {
        this.tahunPerakitan = tahunPerakitan;
    }

    public PitType getPitType() {
        return pitType;
    }

    public void setPitType(PitType pitType) {
        this.pitType = pitType;
    }

    public Integer getPitNumber() {
        return pitNumber;
    }

    public void setPitNumber(Integer pitNumber) {
        this.pitNumber = pitNumber;
    }


    public void setListPromo(Set<PromoWorkOrder> listPromo) {
        if(listPromo == null){
            this.listPromo = listPromo;
        }else {
            if(listPromo.isEmpty()){
                this.listPromo.addAll(listPromo);
            }else {
                this.listPromo.retainAll(listPromo);
                this.listPromo.addAll(listPromo);
            }
        }
    }

    public String getMechanicName() {
        return mechanicName;
    }

    public void setMechanicName(String mechanicName) {
        this.mechanicName = mechanicName;
    }

    public String getUnitTypeName() {
        return unitTypeName;
    }

    public void setUnitTypeName(String unitTypeName) {
        this.unitTypeName = unitTypeName;
    }

    public String getUnitCustomerName() {
        return unitCustomerName;
    }

    public void setUnitCustomerName(String unitCustomerName) {
        this.unitCustomerName = unitCustomerName;
    }

    public CreationalSpecification getWoCreational() {
        return woCreational;
    }

    public void setWoCreational(CreationalSpecification woCreational) {
        this.woCreational = woCreational;
    }

    public String getUnitTypeColor() {
        return unitTypeColor;
    }

    public void setUnitTypeColor(String unitTypeColor) {
        this.unitTypeColor = unitTypeColor;
    }

    public String getSignPhotoCustomer() {
        return signPhotoCustomer;
    }

    public void setSignPhotoCustomer(String signPhotoCustomer) {
        this.signPhotoCustomer = signPhotoCustomer;
    }

    public String getSignPhotoSA() {
        return signPhotoSA;
    }

    public void setSignPhotoSA(String signPhotoSA) {
        this.signPhotoSA = signPhotoSA;
    }

    public String getPackageServiceID() {
        return packageServiceID;
    }

    public void setPackageServiceID(String packageServiceID) {
        this.packageServiceID = packageServiceID;
    }

//    public PaymentStatus getPaymentStatus() {
//        return paymentStatus;
//    }

//    public void setPaymentStatus(PaymentStatus paymentStatus) {
//        this.paymentStatus = paymentStatus;
//    }

    public String getBookingID() {
        return bookingID;
    }

    public void setBookingID(String bookingID) {
        this.bookingID = bookingID;
    }

    public String getKeterangan() {
        return keterangan;
    }

    public void setKeterangan(String keterangan) {
        this.keterangan = keterangan;
    }

    public String getLocation() {
        return location;
    }

    public void setLocation(String location) {
        this.location = location;
    }

    public String getSoID() {
        return soID;
    }

    public void setSoID(String soID) {
        this.soID = soID;
    }

    public String getUnitCategoryID() {
        return unitCategoryID;
    }

    public void setUnitCategoryID(String unitCategoryID) {
        this.unitCategoryID = unitCategoryID;
    }

    public String getUnitCategoryName() {
        return unitCategoryName;
    }

    public void setUnitCategoryName(String unitCategoryName) {
        this.unitCategoryName = unitCategoryName;
    }

    public String getUnitSeriesID() {
        return unitSeriesID;
    }

    public void setUnitSeriesID(String unitSeriesID) {
        this.unitSeriesID = unitSeriesID;
    }

    public String getUnitSeriesName() {
        return unitSeriesName;
    }

    public void setUnitSeriesName(String unitSeriesName) {
        this.unitSeriesName = unitSeriesName;
    }

    public String getPromoCode() {
        return promoCode;
    }

    public void setPromoCode(String promoCode) {
        this.promoCode = promoCode;
    }

    public String getModifiedTrigger() {
        return modifiedTrigger;
    }

    public void setModifiedTrigger(String modifiedTrigger) {
        this.modifiedTrigger = modifiedTrigger;
    }

    public String getInvoice() {
        return invoice;
    }

    public void setInvoice(String invoice) {
        this.invoice = invoice;
    }

    public String getInvoiceUrl() {
        return invoiceUrl;
    }

    public void setInvoiceUrl(String invoiceUrl) {
        this.invoiceUrl = invoiceUrl;
    }

    public String getUnitEntrySource() {
        return unitEntrySource;
    }

    public void setUnitEntrySource(String unitEntrySource) {
        this.unitEntrySource = unitEntrySource;
    }

    public String getNoBukuC2() {
        return noBukuC2;
    }

    public void setNoBukuC2(String noBukuC2) {
        this.noBukuC2 = noBukuC2;
    }

    public String getRekomendasiSA() {
        return rekomendasiSA;
    }

    public void setRekomendasiSA(String rekomendasiSA) {
        this.rekomendasiSA = rekomendasiSA;
    }

    public UnitOwnershipStatus getUnitOwnership() {
        return unitOwnership;
    }

    public void setUnitOwnership(UnitOwnershipStatus unitOwnership) {
        this.unitOwnership = unitOwnership;
    }

    public Double getTotalBookingFee() {
        return totalBookingFee;
    }

    public void setTotalBookingFee(Double totalBookingFee) {
        this.totalBookingFee = totalBookingFee;
    }

    public String getTipeKPB() {
        return tipeKPB;
    }

    public void setTipeKPB(String tipeKPB) {
        this.tipeKPB = tipeKPB;
    }

    public Integer getCountPrint() {
        return countPrint;
    }

    public void setCountPrint(Integer countPrint) {
        this.countPrint = countPrint;
    }

    public String getProsesServicePUD() {
        return prosesServicePUD;
    }

    public void setProsesServicePUD(String prosesServicePUD) {
        this.prosesServicePUD = prosesServicePUD;
    }

    public String getCsID() {
        return csID;
    }

    public void setCsID(String csID) {
        this.csID = csID;
    }


    public Boolean getSpecialCase() {
        return specialCase;
    }

    public void setSpecialCase(Boolean specialCase) {
        this.specialCase = specialCase;
    }

    public String getReffNumber() {
        return reffNumber;
    }

    public void setReffNumber(String reffNumber) {
        this.reffNumber = reffNumber;
    }

    public String getPosServiceID() {
        return posServiceID;
    }

    public void setPosServiceID(String posServiceID) {
        this.posServiceID = posServiceID;
    }

    public ServiceLocationType getServiceLocationType() {
        return serviceLocationType;
    }

    public void setServiceLocationType(ServiceLocationType serviceLocationType) {
        this.serviceLocationType = serviceLocationType;
    }

    public String getAddress() {
        return address;
    }

    public void setAddress(String address) {
        this.address = address;
    }

    public String getKabkotID() {
        return kabkotID;
    }

    public void setKabkotID(String kabkotID) {
        this.kabkotID = kabkotID;
    }

    public String getKecamatanID() {
        return kecamatanID;
    }

    public void setKecamatanID(String kecamatanID) {
        this.kecamatanID = kecamatanID;
    }

    public String getKelurahanID() {
        return kelurahanID;
    }

    public void setKelurahanID(String kelurahanID) {
        this.kelurahanID = kelurahanID;
    }

    public String getProvinceID() {
        return provinceID;
    }

    public void setProvinceID(String provinceID) {
        this.provinceID = provinceID;
    }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = 17 * hash + Objects.hashCode(this.version);
        hash = 17 * hash + Objects.hashCode(this.woID);
        hash = 17 * hash + Objects.hashCode(this.ahassID);
        hash = 17 * hash + Objects.hashCode(this.estimasiMulai);
        hash = 17 * hash + Objects.hashCode(this.estimasiSelesai);
        hash = 17 * hash + Objects.hashCode(this.noPolisi);
        hash = 17 * hash + Objects.hashCode(this.woNoRangka);
        hash = 17 * hash + Objects.hashCode(this.woNoMesin);
        hash = 17 * hash + Objects.hashCode(this.unitCustomerID);
        hash = 17 * hash + Objects.hashCode(this.unitCustomerName);
        hash = 17 * hash + Objects.hashCode(this.customerAttendeeID);
        hash = 17 * hash + Objects.hashCode(this.kmMotor);
        hash = 17 * hash + Objects.hashCode(this.suggestedKm);
        hash = 17 * hash + Objects.hashCode(this.fuelMeter);
        hash = 17 * hash + Objects.hashCode(this.alasanDatang);
        hash = 17 * hash + Objects.hashCode(this.listComplaint);
        hash = 17 * hash + Objects.hashCode(this.analisa);
        hash = 17 * hash + Objects.hashCode(this.typeJasa);
        hash = 17 * hash + Objects.hashCode(this.listJasa);
        hash = 17 * hash + Objects.hashCode(this.totalHargaJasa);
        hash = 17 * hash + Objects.hashCode(this.totalDiscountJasa);
        hash = 17 * hash + Objects.hashCode(this.totalWaktuJasa);
        hash = 17 * hash + Objects.hashCode(this.listPart);
        hash = 17 * hash + Objects.hashCode(this.totalHargaSparepart);
        hash = 17 * hash + Objects.hashCode(this.totalDiscountSparepart);
        hash = 17 * hash + Objects.hashCode(this.woLastKnownState);
        hash = 17 * hash + Objects.hashCode(this.woLastKnownStatus);
        hash = 17 * hash + Objects.hashCode(this.woDisclaimerType);
        hash = 17 * hash + Objects.hashCode(this.sparepartDibawaCustomer);
        hash = 17 * hash + Objects.hashCode(this.confirmNoTelp);
        hash = 17 * hash + Objects.hashCode(this.listPromo);
        hash = 17 * hash + Objects.hashCode(this.queueNumber);
        hash = 17 * hash + Objects.hashCode(this.mechanicID);
        hash = 17 * hash + Objects.hashCode(this.mechanicName);
        hash = 17 * hash + Objects.hashCode(this.unitCategoryID);
        hash = 17 * hash + Objects.hashCode(this.unitCategoryName);
        hash = 17 * hash + Objects.hashCode(this.unitSeriesID);
        hash = 17 * hash + Objects.hashCode(this.unitSeriesName);
        hash = 17 * hash + Objects.hashCode(this.unitTypeID);
        hash = 17 * hash + Objects.hashCode(this.unitTypeName);
        hash = 17 * hash + Objects.hashCode(this.unitTypeColor);
        hash = 17 * hash + Objects.hashCode(this.tahunPerakitan);
        hash = 17 * hash + Objects.hashCode(this.saranMekanik);
        hash = 17 * hash + Objects.hashCode(this.pitType);
        hash = 17 * hash + Objects.hashCode(this.pitNumber);
        hash = 17 * hash + Objects.hashCode(this.woDate);
        hash = 17 * hash + Objects.hashCode(this.signPhotoCustomer);
        hash = 17 * hash + Objects.hashCode(this.signPhotoSA);
        hash = 17 * hash + Objects.hashCode(this.packageServiceID);
        hash = 17 * hash + Objects.hashCode(this.bookingID);
        hash = 17 * hash + Objects.hashCode(this.keterangan);
        hash = 17 * hash + Objects.hashCode(this.location);
        hash = 17 * hash + Objects.hashCode(this.soID);
        hash = 17 * hash + Objects.hashCode(this.promoCode);
        hash = 17 * hash + Objects.hashCode(this.modifiedTrigger);
        hash = 17 * hash + Objects.hashCode(this.invoice);
        hash = 17 * hash + Objects.hashCode(this.woCreational);
        hash = 17 * hash + Objects.hashCode(this.noBukuC2);
        hash = 17 * hash + Objects.hashCode(this.unitEntrySource);
        hash = 17 * hash + Objects.hashCode(this.rekomendasiSA);
        hash = 17 * hash + Objects.hashCode(this.unitOwnership);
        hash = 17 * hash + Objects.hashCode(this.totalBookingFee);
        hash = 17 * hash + Objects.hashCode(this.tipeKPB);
        hash = 17 * hash + Objects.hashCode(this.countPrint);
        hash = 17 * hash + Objects.hashCode(this.prosesServicePUD);
        hash = 17 * hash + Objects.hashCode(this.specialCase);

        hash = 17 * hash + Objects.hashCode(this.serviceLocationType);
        hash = 17 * hash + Objects.hashCode(this.posServiceID);
        hash = 17 * hash + Objects.hashCode(this.address);
        hash = 17 * hash + Objects.hashCode(this.kabkotID);
        hash = 17 * hash + Objects.hashCode(this.kecamatanID);
        hash = 17 * hash + Objects.hashCode(this.kelurahanID);
        hash = 17 * hash + Objects.hashCode(this.provinceID);

        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final WorkOrder other = (WorkOrder) obj;
        if (!Objects.equals(this.woID, other.woID)) {
            return false;
        }
        if (!Objects.equals(this.ahassID, other.ahassID)) {
            return false;
        }
        if (!Objects.equals(this.version, other.version)) {
            return false;
        }
        if (!Objects.equals(this.noPolisi, other.noPolisi)) {
            return false;
        }
        if (!Objects.equals(this.woNoRangka, other.woNoRangka)) {
            return false;
        }
        if (!Objects.equals(this.woNoMesin, other.woNoMesin)) {
            return false;
        }
        if (!Objects.equals(this.unitCustomerID, other.unitCustomerID)) {
            return false;
        }
        if (!Objects.equals(this.unitCustomerName, other.unitCustomerName)) {
            return false;
        }
        if (!Objects.equals(this.customerAttendeeID, other.customerAttendeeID)) {
            return false;
        }
        if (!Objects.equals(this.alasanDatang, other.alasanDatang)) {
            return false;
        }
        if (!Objects.equals(this.analisa, other.analisa)) {
            return false;
        }
        if (!Objects.equals(this.typeJasa, other.typeJasa)) {
            return false;
        }
        if (!Objects.equals(this.confirmNoTelp, other.confirmNoTelp)) {
            return false;
        }
        if (!Objects.equals(this.mechanicID, other.mechanicID)) {
            return false;
        }
        if (!Objects.equals(this.mechanicName, other.mechanicName)) {
            return false;
        }
        if (!Objects.equals(this.unitCategoryID, other.unitCategoryID)) {
            return false;
        }
        if (!Objects.equals(this.unitCategoryName, other.unitCategoryName)) {
            return false;
        }
        if (!Objects.equals(this.unitSeriesID, other.unitSeriesID)) {
            return false;
        }
        if (!Objects.equals(this.unitSeriesName, other.unitSeriesName)) {
            return false;
        }
        if (!Objects.equals(this.unitTypeID, other.unitTypeID)) {
            return false;
        }
        if (!Objects.equals(this.unitTypeName, other.unitTypeName)) {
            return false;
        }
        if (!Objects.equals(this.unitTypeColor, other.unitTypeColor)) {
            return false;
        }
        if (!Objects.equals(this.saranMekanik, other.saranMekanik)) {
            return false;
        }
        if (!Objects.equals(this.signPhotoCustomer, other.signPhotoCustomer)) {
            return false;
        }
        if (!Objects.equals(this.signPhotoSA, other.signPhotoSA)) {
            return false;
        }
        if (!Objects.equals(this.packageServiceID, other.packageServiceID)) {
            return false;
        }
        if (!Objects.equals(this.bookingID, other.bookingID)) {
            return false;
        }
        if (!Objects.equals(this.keterangan, other.keterangan)) {
            return false;
        }
        if (!Objects.equals(this.location, other.location)) {
            return false;
        }
        if (!Objects.equals(this.soID, other.soID)) {
            return false;
        }
        if (!Objects.equals(this.estimasiMulai, other.estimasiMulai)) {
            return false;
        }
        if (!Objects.equals(this.estimasiSelesai, other.estimasiSelesai)) {
            return false;
        }
        if (!Objects.equals(this.kmMotor, other.kmMotor)) {
            return false;
        }
        if (!Objects.equals(this.suggestedKm, other.suggestedKm)) {
            return false;
        }
        if (!Objects.equals(this.fuelMeter, other.fuelMeter)) {
            return false;
        }
        if (!Objects.equals(this.listComplaint, other.listComplaint)) {
            return false;
        }
        if (!Objects.equals(this.listJasa, other.listJasa)) {
            return false;
        }
        if (!Objects.equals(this.totalHargaJasa, other.totalHargaJasa)) {
            return false;
        }
        if (!Objects.equals(this.totalDiscountJasa, other.totalDiscountJasa)) {
            return false;
        }
        if (!Objects.equals(this.totalWaktuJasa, other.totalWaktuJasa)) {
            return false;
        }
        if (!Objects.equals(this.listPart, other.listPart)) {
            return false;
        }
        if (!Objects.equals(this.totalHargaSparepart, other.totalHargaSparepart)) {
            return false;
        }
        if (!Objects.equals(this.totalDiscountSparepart, other.totalDiscountSparepart)) {
            return false;
        }
        if (this.woLastKnownState != other.woLastKnownState) {
            return false;
        }
        if (this.woLastKnownStatus != other.woLastKnownStatus) {
            return false;
        }
        if (this.woDisclaimerType != other.woDisclaimerType) {
            return false;
        }
        if (!Objects.equals(this.sparepartDibawaCustomer, other.sparepartDibawaCustomer)) {
            return false;
        }
        if (!Objects.equals(this.listPromo, other.listPromo)) {
            return false;
        }
        if (!Objects.equals(this.queueNumber, other.queueNumber)) {
            return false;
        }
        if (!Objects.equals(this.tahunPerakitan, other.tahunPerakitan)) {
            return false;
        }
        if (this.pitType != other.pitType) {
            return false;
        }
        if (!Objects.equals(this.pitNumber, other.pitNumber)) {
            return false;
        }
        if (!Objects.equals(this.woDate, other.woDate)) {
            return false;
        }
        if (!Objects.equals(this.promoCode, other.promoCode)) {
            return false;
        }
        if (!Objects.equals(this.modifiedTrigger, other.modifiedTrigger)) {
            return false;
        }
        if (!Objects.equals(this.invoice, other.invoice)) {
            return false;
        }
        if (!Objects.equals(this.woCreational, other.woCreational)) {
            return false;
        }
        if (!Objects.equals(this.noBukuC2, other.noBukuC2)) {
            return false;
        }
        if (!Objects.equals(this.unitEntrySource, other.unitEntrySource)) {
            return false;
        }
        if (!Objects.equals(this.rekomendasiSA, other.rekomendasiSA)) {
            return false;
        }
        if (!Objects.equals(this.unitOwnership, other.unitOwnership)) {
            return false;
        }
        if (!Objects.equals(this.totalBookingFee, other.totalBookingFee)) {
            return false;
        }
        if (!Objects.equals(this.tipeKPB, other.tipeKPB)) {
            return false;
        }
        if (!Objects.equals(this.countPrint, other.countPrint)) {
            return false;
        }
        if (!Objects.equals(this.prosesServicePUD, other.prosesServicePUD)) {
            return false;
        }

        if (!Objects.equals(this.specialCase, other.specialCase)) {
            return false;
        }
        if (!Objects.equals(this.posServiceID, other.posServiceID)) {
            return false;
        }
        if (!Objects.equals(this.address, other.address)) {
            return false;
        }
        if (!Objects.equals(this.kabkotID, other.kabkotID)) {
            return false;
        }
        if (!Objects.equals(this.kecamatanID, other.kecamatanID)) {
            return false;
        }
        if (!Objects.equals(this.kelurahanID, other.kelurahanID)) {
            return false;
        }
        if (!Objects.equals(this.provinceID, other.provinceID)) {
            return false;
        }
        if (!Objects.equals(this.serviceLocationType, other.serviceLocationType)) {
            return false;
        }
        return true;
    }

    @Override
    public boolean sameIdentityAs(WorkOrder t) {
        return this.equals(t);
    }
}
